//   Copyright 2007, 2008 Nicolas Dubuit
//
//     This file is part of Immix.
//
//     Immix is free software: you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation, either version 3 of the License, or
//     (at your option) any later version.
//
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
//
//     You should have received a copy of the GNU General Public License
//     along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <fftw3.h>
#include <QImage>
#include <QMutex>

#ifndef CORRELATOR_H
#define CORRELATOR_H

class Correlator
{
    // Mutex, for thread-safety.
    // fftw functions share data, and the mutex should protect that.
    static QMutex fftwMutex ;

    // Image size, and image base size.
    int FineFactor, ImgSz ;

    // Size of correlation data.
    int CorSz ;

    // Base data.
    fftw_complex *targGray ;	
	fftw_complex *refGray ;	

    // Data in fourier space.
	fftw_complex *fRefGray ;	
	fftw_complex *fTargGray ;	

    // Correlation
    /// Caution ! This is just a copy of targGray (for memory usage reasons)
    /// This means that targGray will be overwritten during correlation calculation.
	fftw_complex *Correlation ;

    // FFT plans.
    fftw_plan pTargGray ;
    fftw_plan pRefGray ;
	fftw_plan pBackTarg ;

    // set data.
    void set_data (QImage&, fftw_complex*) ;
    void set_data_square (QImage&, fftw_complex*) ;
    void set_window_data (fftw_complex*) ;

public :
    Correlator (int, int) ;
    ~Correlator (void) ;

    void setRefData (QImage&) ;
    void setTargetData (QImage&) ;
    void setRefDataSquare (QImage&) ;
    void setTargetDataSquare (QImage&) ;
    void setWindowRefData (void) ;
    void setWindowTargetData (void) ;

    void calculateCorrelation (void) ;

    void DivideByRootOf (Correlator& divisor) ;

    void findCorrelMax (double& dXmax, double& dYmax, 
                        double& corMin, double& corMax, 
                        double& corIMin, double& corIMax) ;

    void freeRefMemory (void) ;
    void freeTargAndCorrelMemory (void) ;
    void freeFourierMemory (void) ;
} ;

#endif 
